import 'package:tencent_cloud_chat/log/tencent_cloud_chat_log.dart';
import 'package:tencent_cloud_chat/tencent_cloud_chat.dart';
import 'package:tencent_cloud_chat_sdk/tencent_im_sdk_plugin.dart';

class TencentCloudChatGroupSDK {
  // Private constructor to implement the singleton pattern.
  TencentCloudChatGroupSDK._internal();

  // Factory constructor that returns the singleton instance of TencentCloudChatMessageSDK.
  factory TencentCloudChatGroupSDK() => _instance;
  static final TencentCloudChatGroupSDK _instance =
      TencentCloudChatGroupSDK._internal();

  Future<List<V2TimGroupInfoResult>?> getGroupsInfo({
    required List<String> groupIDList,
  }) async {
    final res = await TencentImSDKPlugin.v2TIMManager
        .getGroupManager()
        .getGroupsInfo(groupIDList: groupIDList);
    if (res.code == 0) {
      return res.data;
    } else {
      var componentName =
          runtimeType.toString().replaceAll("TencentCloudChat", "");
      TencentCloudChat.logInstance.console(
        componentName: componentName,
        logs: "getGroupsInfo - ${res.desc}",
        logLevel: TencentCloudChatLogLevel.error,
      );
      return null;
    }
  }

  Future<V2TimValueCallback<V2TimGroupMemberInfoResult>?> getGroupMemberList({
    required String groupID,
    required GroupMemberFilterTypeEnum filter,
    required String nextSeq,
    int count = 15,
    int offset = 0,
  }) async {
    final res = await TencentImSDKPlugin.v2TIMManager
        .getGroupManager()
        .getGroupMemberList(
            groupID: groupID,
            filter: filter,
            nextSeq: nextSeq,
            count: count,
            offset: offset);
    if (res.code == 0) {
      return res;
    }
    var componentName =
        runtimeType.toString().replaceAll("TencentCloudChat", "");
    TencentCloudChat.logInstance.console(
      componentName: componentName,
      logs: "getGroupMemberList - ${res.desc}",
      logLevel: TencentCloudChatLogLevel.error,
    );
    return null;
  }
}
