enum TencentCloudChatComponents {
  conversation,
  message,
  search,
  contact,
  userProfile,
  groupProfile
}

/// A class that defines route names for TencentCloudChat.
///
/// This class provides a set of constants that represent the route names
/// for various components in the Chat UIKit. These route names can be used
/// for navigation between components.
class TencentCloudChatRouteNames {
  /// Route name for the conversation component.
  static const conversation = 'tuikit:///conversation/conversation.component';

  /// Route name for the message component.
  static const message = 'tuikit:///message/message.component';

  /// Route name for the message info component.
  static const messageInfo = 'tuikit:///message/messageInfo.component';

  /// Route name for the global search component.
  static const globalSearch = 'tuikit:///search/globalSearch.component';

  /// Route name for the message search component.
  static const messageSearch = 'tuikit:///search/messageSearch.component';

  /// Route name for the user profile component.
  static const userProfile = 'tuikit:///userProfile/userProfile.component';

  /// Route name for the user profile edit component.
  static const userProfileEdit =
      'tuikit:///userProfile/userProfileEdit.component';

  /// Route name for the group profile component.
  static const groupProfile = 'tuikit:///groupProfile/groupProfile.component';

  /// Route name for the group member list component.
  static const groupMemberList =
      'tuikit:///groupProfile/groupMemberList.component';

  /// Route name for the group notice component.
  static const groupNotice = 'tuikit:///groupProfile/groupNotice.component';

  /// Route name for the group management component.
  static const groupManagement =
      'tuikit:///groupProfile/groupManagement.component';

  /// Route name for the contacts component.
  static const contact = 'tuikit:///contact/contacts.component';

  /// Route name for the add contact component.
  static const addContact = 'tuikit:///contact/addContact.component';

  /// Route name for the add group component.
  static const addGroup = 'tuikit:///contact/addGroup.component';

  /// Route name for the new contact application component.
  static const newContactApplication =
      'tuikit:///contact/newContactApplication.component';

  /// Route name for the new contact application detail component.
  static const newContactApplicationDetail =
      'tuikit:///contact/newContactApplicationDetail.component';

  /// Route name for the group list component.
  static const groupList = 'tuikit:///contact/groupList.component';

  /// Route name for the block list component
  static const blockList = 'tuikit:///contact/blockList.component';

  /// Route name for the friend information component
  static const friendInfo = 'tuikit:///contact/friendInfo.component';

  /// Route name for the group application list component
  static const groupApplication =
      'tuikit:///contact/groupApplication.component';
  static const settings = 'tuikit:///settings/settings.component';
  static const settingsInfo = 'tuikit:///settings/settingsInfo.component';
  static const settingsAbout = 'tuikit:///settings/settingsAbout.component';
  static const homePage = 'tuikit:///homePage.component';
  static const loginPage = 'tuikit:///loginPage.component';
}
