// ignore_for_file: depend_on_referenced_packages

import 'dart:async';
import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:exif/exif.dart';
import 'package:flutter/foundation.dart';
import 'package:intl/intl.dart';
import 'package:tencent_cloud_chat/tencent_cloud_chat.dart';
import 'package:tencent_cloud_chat/utils/tencent_cloud_chat_message_calling_message/tencent_cloud_chat_message_calling_message.dart';

class ImageExifInfo {
  final double width;
  final double height;
  final bool isRotate;

  ImageExifInfo({
    required this.height,
    required this.isRotate,
    required this.width,
  });
}

class TencentCloudChatUtils {
  static String localizedDateString(int timestamp) {
    // Convert the timestamp (seconds) to a DateTime object
    DateTime date = DateTime.fromMillisecondsSinceEpoch(timestamp * 1000);

    // Get the current year
    int currentYear = DateTime.now().year;

    // Choose the date format based on whether the date is in the current year
    DateFormat dateFormat = date.year == currentYear ? DateFormat.MMMMd() : DateFormat.yMMMMd();

    // Format the date using the selected date format
    String dateString = dateFormat.format(date);

    return dateString;
  }

  /// Returns a formatted string representation of the current date and time.
  ///
  /// [dateTime] (optional) represents the date and time to format. If not provided, the current date and time will be used.
  ///
  /// Returns a string in the format "yyyy-MM-dd hh:mm:ss a".
  ///
  /// This function is used to format the date and time for logging purposes.
  ///
  /// Example:
  /// ```dart
  /// void main() {
  ///   /// Get the current formatted date and time
  ///   String formattedDateTime = getFormattedTimeString();
  ///
  ///   print("Current date and time: $formattedDateTime");
  /// }
  /// ```
  static String getFormattedTimeString({
    DateTime? dateTime,
  }) {
    // If dateTime is not provided, use the current date and time.
    dateTime ??= DateTime.now();

    // Create a DateFormat object for formatting the date and time.
    final dateFormat = DateFormat('yyyy-MM-dd hh:mm:ss a');

    // Format the DateTime object as a localized string and return it.
    return dateFormat.format(dateTime);
  }

  /// This function checks if the given text is not null and not empty.
  /// Returns the text if it's not null and not empty, otherwise returns null.
  /// Example: checkString("Hello") returns "Hello".
  static String? checkString(String? text) {
    return (text != null && text.isNotEmpty) ? text : null;
  }

  /// This function checks if the given text is not null, not empty, and does not contain spaces.
  /// Returns the text if it meets the conditions, otherwise returns null.
  /// Example: checkStringWithoutSpace("Hello World") returns null.
  static String? checkStringWithoutSpace(String? text) {
    if (text == null || text.trim().isEmpty || text.contains(' ')) {
      return null;
    }
    return text;
  }

  /// This function returns the MIME type based on the given file extension.
  /// Example: getFileType("pdf") returns "application/pdf".
  static String getFileType(String fileType) {
    switch (fileType) {
      case "3gp":
        return "video/3gpp";
      case "torrent":
        return "application/x-bittorrent";
      case "kml":
        return "application/vnd.google-earth.kml+xml";
      case "gpx":
        return "application/gpx+xml";
      case "asf":
        return "video/x-ms-asf";
      case "avi":
        return "video/x-msvideo";
      case "bin":
      case "class":
      case "exe":
        return "application/octet-stream";
      case "bmp":
        return "image/bmp";
      case "c":
        return "text/plain";
      case "conf":
        return "text/plain";
      case "cpp":
        return "text/plain";
      case "doc":
        return "application/msword";
      case "docx":
        return "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
      case "xls":
      case "csv":
        return "application/vnd.ms-excel";
      case "xlsx":
        return "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
      case "gif":
        return "image/gif";
      case "gtar":
        return "application/x-gtar";
      case "gz":
        return "application/x-gzip";
      case "h":
        return "text/plain";
      case "htm":
        return "text/html";
      case "html":
        return "text/html";
      case "jar":
        return "application/java-archive";
      case "java":
        return "text/plain";
      case "jpeg":
        return "image/jpeg";
      case "jpg":
        return "image/jpeg";
      case "js":
        return "application/x-javascript";
      case "log":
        return "text/plain";
      case "m3u":
        return "audio/x-mpegurl";
      case "m4a":
        return "audio/mp4a-latm";
      case "m4b":
        return "audio/mp4a-latm";
      case "m4p":
        return "audio/mp4a-latm";
      case "m4u":
        return "video/vnd.mpegurl";
      case "m4v":
        return "video/x-m4v";
      case "mov":
        return "video/quicktime";
      case "mp2":
        return "audio/x-mpeg";
      case "mp3":
        return "audio/x-mpeg";
      case "mp4":
        return "video/mp4";
      case "mpc":
        return "application/vnd.mpohun.certificate";
      case "mpe":
        return "video/mpeg";
      case "mpeg":
        return "video/mpeg";
      case "mpg":
        return "video/mpeg";
      case "mpg4":
        return "video/mp4";
      case "mpga":
        return "audio/mpeg";
      case "msg":
        return "application/vnd.ms-outlook";
      case "ogg":
        return "audio/ogg";
      case "pdf":
        return "application/pdf";
      case "png":
        return "image/png";
      case "pps":
        return "application/vnd.ms-powerpoint";
      case "ppt":
        return "application/vnd.ms-powerpoint";
      case "pptx":
        return "application/vnd.openxmlformats-officedocument.presentationml.presentation";
      case "prop":
        return "text/plain";
      case "rc":
        return "text/plain";
      case "rmvb":
        return "audio/x-pn-realaudio";
      case "rtf":
        return "application/rtf";
      case "sh":
        return "text/plain";
      case "tar":
        return "application/x-tar";
      case "tgz":
        return "application/x-compressed";
      case "txt":
        return "text/plain";
      case "wav":
        return "audio/x-wav";
      case "wma":
        return "audio/x-ms-wma";
      case "wmv":
        return "audio/x-ms-wmv";
      case "wps":
        return "application/vnd.ms-works";
      case "xml":
        return "text/plain";
      case "z":
        return "application/x-compress";
      case "zip":
        return "application/x-zip-compressed";
      default:
        return "*/*";
    }
  }

  /// Formats a given timestamp (in seconds) into a time string in the format of "11:00 AM".
  ///
  /// This method takes an integer [timestamp] representing the number of seconds since the Unix epoch
  /// (January 1, 1970 at 00:00:00 UTC) and returns a formatted time string.
  ///
  /// The returned string will be in the format of "hh:mm a", where "hh" is the hour (00-11),
  /// "mm" is the minute (00-59), and "a" is either "AM" or "PM".
  ///
  /// Example:
  /// ```
  /// int timestamp = 1635504600; // Represents "2021-10-29 11:00:00"
  /// String formattedTime = formatTimestampToTime(timestamp);
  /// print(formattedTime); // Output: "11:00 AM"
  /// ```
  static String formatTimestampToTime(int timestamp) {
    // Convert the timestamp (in seconds) to a DateTime object.
    final dateTime = DateTime.fromMillisecondsSinceEpoch(timestamp * 1000);

    // Create a DateFormat object for formatting the time.
    final timeFormat = DateFormat.jm();

    // Format the DateTime object as a time string and return it.

    return timeFormat.format(dateTime);
  }

  static String getMessageSummary({V2TimMessage? message, int? messageReceiveOption, int? unreadCount, String? draftText}) {
    String text = "";

    if (message != null) {
      switch (message.elemType) {
        case MessageElemType.V2TIM_ELEM_TYPE_TEXT:
          if (message.textElem != null) {
            text = message.textElem!.text ?? "";
          }
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_CUSTOM:
          text = handleCustomMessage(message);
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_SOUND:
          text = "[${tL10n.audio}]";
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_FACE:
          text = '[${tL10n.sticker}]';
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_FILE:
          text = '[${tL10n.file}]';
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_GROUP_TIPS:
          text = "[${tL10n.groupTips}]${buildGroupTipsText(message.groupTipsElem)}";
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_IMAGE:
          text = '[${tL10n.image}]';
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_VIDEO:
          text = '[${tL10n.video}]';
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_LOCATION:
          text = '[${tL10n.location}]';
          break;
        case MessageElemType.V2TIM_ELEM_TYPE_MERGER:
          text = '[${tL10n.chatHistory}]';
          break;
        default:
          text = "";
      }
      if (message.status == 4) {
        text = tL10n.messageDeleted;
      }
      if (message.status == 6) {
        text = tL10n.messageRecalled;
      }
      if (messageReceiveOption != 0) {
        if (unreadCount != null) {
          if (unreadCount > 0) {
            text = "[${tL10n.unreadCount(unreadCount)}]$text";
          }
        }
      }
      if (TencentCloudChatUtils.checkString(draftText) != null) {
        text = "";
      }
    }

    return text;
  }

  static bool isVoteMessage(V2TimMessage message) {
    bool isVoteMessage = false;
    V2TimCustomElem? custom = message.customElem;

    if (custom != null) {
      String? data = custom.data;
      if (data != null && data.isNotEmpty) {
        try {
          Map<String, dynamic> mapData = json.decode(data);
          if (mapData["businessID"] == "group_poll") {
            isVoteMessage = true;
          }
        } catch (err) {
          // err
        }
      }
    }
    return isVoteMessage;
  }

  static String buildGroupTipsText(V2TimGroupTipsElem? tips) {
    String res = "";
    if (tips != null) {
      int type = tips.type;
      List<V2TimGroupChangeInfo?> groupChangeInfo = tips.groupChangeInfoList ?? [];
      List<V2TimGroupMemberChangeInfo?> memberChangeInfo = (tips.memberChangeInfoList ?? []);
      List<V2TimGroupMemberInfo?> memberList = (tips.memberList ?? []);
      V2TimGroupMemberInfo opMember = tips.opMember;

      String membersDisplayText = memberList.map((e) => TencentCloudChatUtils.getDisplayNameByV2TimGroupMemberInfo(TencentCloudChatUtils.v2TimGroupMemberInfo2V2TimGroupMemberFullInfo(e))).join(",");
      String opMemberDisplayText = TencentCloudChatUtils.getDisplayNameByV2TimGroupMemberInfo(TencentCloudChatUtils.v2TimGroupMemberInfo2V2TimGroupMemberFullInfo(opMember));

      switch (type) {
        case 0:
          res = tL10n.unknownGroupTips;
          break;
        case 1:
          res = tL10n.memberJoinedGroup(membersDisplayText);
          break;
        case 2:
          res = tL10n.opInvitedToGroup(membersDisplayText, opMemberDisplayText);
          break;
        case 3:
          res = tL10n.memberLeftGroup(membersDisplayText);
          break;
        case 4:
          res = tL10n.opRemovedFromGroup(membersDisplayText, opMemberDisplayText);
          break;
        case 5:
          res = tL10n.opPromotedToAdmin(membersDisplayText, opMemberDisplayText);
          break;
        case 6:
          res = tL10n.opRevokedAdmin(membersDisplayText, opMemberDisplayText);
          break;
        case 7:
          res = tL10n.opChangedGroupInfo(groupChangeInfo.map((e) => buildGroupChangeInfoText(e)).join(","), opMemberDisplayText);
          break;
        case 9:
          res = tL10n.opChangedMemberInfo(memberChangeInfo.map((e) => buildGroupMemberChangeInfoText(e)).join(","), opMemberDisplayText);
          break;
      }
    }
    return res;
  }

  static final Map<String, Timer> _debounceTimers = {};
  static final Map<String, Timer> _throttleTimers = {};

  /// Debounce function that delays the execution of the [callback] function
  /// by the specified [duration].
  ///
  /// This function is useful when you want to make sure that a certain operation
  /// is not performed too frequently, for example, when listening to input events
  /// or making API calls.
  ///
  /// [callback]: The function to be executed after the debounce duration.
  /// [duration]: The duration to wait before executing the callback function.
  static void debounce(String key, Function() callback, {Duration duration = const Duration(milliseconds: 500)}) {
    _debounceTimers[key]?.cancel();
    _debounceTimers[key] = Timer(duration, callback);
  }

  /// Throttle function that ensures the [callback] function is not called
  /// more often than the specified [duration].
  ///
  /// This function is useful when you want to limit the rate at which an
  /// operation is performed, for example, when listening to scroll events
  /// or updating the UI.
  ///
  /// [callback]: The function to be executed with a throttled rate.
  /// [duration]: The minimum duration between consecutive callback executions.
  static void throttle(String key, Function() callback, {Duration duration = const Duration(milliseconds: 500)}) {
    if (_throttleTimers[key] == null) {
      _throttleTimers[key] = Timer(duration, () {
        callback();
        _throttleTimers.remove(key);
      });
    }
  }

  static bool deepEqual(Object? previous, Object? next) {
    return const DeepCollectionEquality.unordered().equals(previous, next);
  }

  static Future<ImageExifInfo> getImageExifInfoByBuffer({
    required Uint8List fileBuffer,
  }) async {
    final data = await readExifFromBytes(fileBuffer);
    String owidth = data["EXIF ExifImageWidth"]?.printable ?? "200";
    String oheight = data["EXIF ExifImageLength"]?.printable ?? "260";
    bool isRotate = false;
    final rotate = data["Image Orientation"]?.printable ?? "";
    if (rotate.contains("90") || rotate.contains("270")) {
      isRotate = true;
    }
    if (isRotate) {
      (owidth, oheight) = (oheight, owidth);
    }
    return ImageExifInfo(height: double.parse(oheight), width: double.parse(owidth), isRotate: isRotate);
  }

  static addDataToStringFiled({
    required String key,
    required String value,
    required String currentString,
  }) {
    Map<String, String> currentObj;
    try {
      currentObj = json.decode(currentString);
    } catch (err) {
      currentObj = {};
    }
    currentObj[key] = value;
    currentString = json.encode(currentObj);

    return currentString;
  }

  static String getDisplayNameByV2TimGroupMemberInfo(
    V2TimGroupMemberFullInfo? info,
  ) {
    String res = "";
    if (info == null) {
      return "";
    }

    if (checkString(info.userID) == null) {
      return res;
    }
    if (checkString(info.friendRemark) != null) {
      return info.friendRemark!;
    }
    if (checkString(info.nameCard) != null) {
      return info.nameCard!;
    }
    if (checkString(info.nickName) != null) {
      return info.nickName!;
    }
    return info.userID;
  }

  static String buildGroupChangeInfoText(V2TimGroupChangeInfo? info) {
    if (info == null) {
      return "";
    }
    String res = "";
    int type = info.type ?? 0;
    switch (type) {
      case 0:
        res = "";
        break;
      case 1:
        res = tL10n.changedGroupNameTo(info.value ?? "empty");
        break;
      case 2:
        res = tL10n.changedGroupDescriptionTo(info.value ?? "empty");
        break;
      case 3:
        res = tL10n.changedGroupAnnouncementTo(info.value ?? "empty");
        break;
      case 4:
        res = tL10n.changedGroupAvatar;
        break;
      case 5:
        res = tL10n.transferredGroupOwnershipTo(info.value ?? "empty");
        break;
      case 6:
        res = tL10n.changedGroupCustomInfo;
        break;
      case 8:
        if (info.boolValue == true) {
          res = tL10n.enabledGroupMute;
        } else {
          res = tL10n.disabledGroupMute;
        }
        res = "";
        break;
      case 10:
        int value = info.intValue ?? 0;
        String conf = "";
        if (value == 0) {
          conf = tL10n.receiveMessages;
        } else if (value == 1) {
          conf = tL10n.doNotReceiveMessages;
        } else {
          conf = tL10n.receiveMessagesWhenOnlineOnly;
        }
        res = tL10n.changedGroupMessageReceptionTo(conf);
        break;
      case 11:
        int value = info.intValue ?? 2;
        String conf = "";
        if (value == 0) {
          conf = tL10n.disallowJoinGroup;
        } else if (value == 1) {
          conf = tL10n.joinGroupNeedApproval;
        } else {
          conf = tL10n.joinGroupDirectly;
        }
        res = tL10n.changedApplyToJoinGroupTo(conf);
        break;
      case 12:
        int value = info.intValue ?? 2;
        String conf = "";
        if (value == 0) {
          conf = tL10n.disallowInviting;
        } else if (value == 1) {
          conf = tL10n.requireApprovalForInviting;
        } else {
          conf = tL10n.joinDirectlyBeenInvited;
        }
        res = tL10n.changedInviteToJoinGroupTo(conf);
        break;
    }
    return res;
  }

  static String buildGroupMemberChangeInfoText(V2TimGroupMemberChangeInfo? info) {
    if (info == null) {
      return "";
    }
    String res = "";
    if (TencentCloudChatUtils.checkString(info.userID) != null) {
      int muteTime = info.muteTime ?? 0;
      String conf = "";
      if (muteTime == 0) {
        conf = tL10n.unmuted;
      } else {
        conf = tL10n.muteTime(muteTime);
      }
      res = "${info.userID} $conf";
    }
    return res;
  }

  static v2TimGroupMemberInfo2V2TimGroupMemberFullInfo(V2TimGroupMemberInfo? info) {
    return V2TimGroupMemberFullInfo.fromJson({
      "faceUrl": info?.faceUrl,
      "friendRemark": info?.friendRemark,
      "nameCard": info?.nameCard,
      "userID": info?.userID ?? "",
    });
  }

  /// Parses the given [jsonString] to extract the messageSender, messageAbstract, and messageID.
  ///
  /// The [jsonString] should be a JSON string with the following structure:
  /// {
  ///   "messageReply": {
  ///     "messageID": "144115216452519330-1698670328-3674128",
  ///     "messageAbstract": "hi, beauty",
  ///     "messageSender": "qwe2203",
  ///     "messageType": 1,
  ///     "version": 1
  ///   }
  /// }
  ///
  /// The function first checks if there is a "messageReply" key in the JSON object.
  /// If it exists, the function extracts the "messageSender", "messageAbstract", and "messageID" keys.
  ///
  /// If the JSON parsing fails or the relevant information is not found,
  /// all values (messageSender, messageAbstract, and messageID) will be returned as null.
  ///
  /// Returns a tuple containing the messageSender, messageAbstract, and messageID.
  static ({String? messageSender, String? messageAbstract, String? messageID}) parseMessageReply(String? jsonString) {
    String? messageSender;
    String? messageAbstract;
    String? messageID;

    if (TencentCloudChatUtils.checkString(jsonString) != null) {
      try {
        Map<String, dynamic> jsonData = json.decode(jsonString!);
        Map<String, dynamic>? messageReply = jsonData["messageReply"];

        if (messageReply != null) {
          messageSender = messageReply["messageSender"] as String?;
          messageAbstract = messageReply["messageAbstract"] as String?;
          messageID = messageReply["messageID"] as String?;

          if (TencentCloudChatUtils.checkString(messageAbstract) != null) {
            final tryParseMessageAbstract = json.decode(messageAbstract!);
            final tryMessageAbstract = tryParseMessageAbstract["summary"];
            if (TencentCloudChatUtils.checkString(tryMessageAbstract) != null) {
              messageAbstract = tryMessageAbstract;
            }
          }
        }
        // ignore: empty_catches
      } catch (e) {}
    }

    return (messageSender: messageSender, messageAbstract: messageAbstract, messageID: messageID);
  }

  static ({String character, int index, bool isAddText}) compareString(String oldText, String newText) {
    final isAddText = newText.length > oldText.length;
    final longerText = isAddText ? newText : oldText;
    final shorterText = isAddText ? oldText : newText;

    int diffIndex = -1;
    for (int i = 0; i < shorterText.length; i++) {
      if (longerText[i] != shorterText[i]) {
        diffIndex = i;
        break;
      }
    }

    if (diffIndex == -1) {
      diffIndex = shorterText.length;
    }

    String characters = longerText.substring(diffIndex, longerText.length - shorterText.length + diffIndex);

    return (
      character: characters,
      index: diffIndex,
      isAddText: isAddText,
    );
  }

  static String handleCustomMessage(V2TimMessage message) {
    final customElem = message.customElem;
    String customLastMsgShow = "[${tL10n.custom}]";

    if (customElem?.data == "group_create") {
      customLastMsgShow = "Group chat created successfully!";
    }
    if (isVoteMessage(message)) {
      customLastMsgShow = "[${tL10n.poll}]";
    }
    final callingMessage = CallingMessage.getCallMessage(customElem);
    if (callingMessage != null) {
      // If it's an end message
      final isCallEnd = CallingMessage.isCallEndExist(callingMessage);
      final isVoiceCall = callingMessage.callType == 1;

      String? callTime = "";

      if (isCallEnd) {
        callTime = CallingMessage.getShowTime(callingMessage.callEnd!);
      }

      customLastMsgShow = isCallEnd ? tL10n.callDuration(callTime ?? "0") : CallingMessage.getActionType(callingMessage);

      customLastMsgShow = isVoiceCall ? "[${tL10n.voiceCall}]: $customLastMsgShow" : "[${tL10n.videoCall}]: $customLastMsgShow";
    }
    if (customLastMsgShow == "[${tL10n.custom}]") {
      debugPrint(message.customElem!.toJson().toString());
    }
    return customLastMsgShow;
  }
}
